package cn.com.ty.lift.business.contract.service;

import cn.com.ty.lift.business.contract.dao.entity.Contracts;
import cn.com.ty.lift.business.contract.dao.entity.Payment;
import cn.com.ty.lift.business.contract.dao.entity.model.request.PaymentRequest;
import cn.com.ty.lift.business.contract.dao.entity.model.response.PaymentDetailResponse;
import cn.com.ty.lift.business.contract.dao.entity.model.response.PaymentListResponse;
import cn.com.ty.lift.business.contract.dao.entity.model.response.PaymentResponse;
import cn.com.ty.lift.business.contract.dao.mapper.PaymentMapper;
import cn.com.ty.lift.common.utils.DateUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author bieao
 * @date 2019/12/11
 * @description 收款业务层
 */
@Service
public class PaymentService extends ServiceImpl<PaymentMapper, Payment> {

    /**
     * @param request 付款列表查询条件
     * @return RestResponse 付款分页列表结果
     * @description 查询付款列表
     * @date 2020/2/20 3:10 下午
     */
    public IPage<PaymentListResponse> maintenanceList(PaymentRequest request){
        return baseMapper.findProjectByCondition(request.getPage(), request);
    }

    /**
     * @param request 付款列表查询条件
     * @return RestResponse 付款分页列表结果
     * @description 查询大修付款列表
     * @date 2020/4/27 11:15 上午
     */
    public IPage<PaymentListResponse> capitalList(PaymentRequest request){
        return baseMapper.findCapitalByCondition(request.getPage(), request);
    }

    /**
     * @param request 付款详情查询条件
     * @return RestResponse 付款详情
     * @description 查询维保付款详情
     * @date 2020/4/28 2:16 下午
     */
    public PaymentResponse maintenanceDetail(PaymentRequest request) {
        PaymentResponse response = new PaymentResponse();
        PaymentDetailResponse detail = baseMapper.queryProjectDetailById(request);
        List<Payment> paymentList = paymentList(request.getId());
        response.setDetail(detail);
        response.setPaymentList(paymentList);
        return response;
    }

    /**
     * @param request 付款详情查询条件
     * @return RestResponse 付款详情
     * @description 查询大修付款详情
     * @date 2020/2/24 10:47 上午
     */
    public PaymentResponse capitalDetail(PaymentRequest request){
        PaymentResponse response = new PaymentResponse();
        PaymentDetailResponse detail = baseMapper.queryCapitalDetailById(request);
        List<Payment> paymentList = paymentList(request.getId());
        response.setPaymentList(paymentList);
        response.setDetail(detail);
        return response;
    }

    /**
     * @param payments 付款信息
     * @param contracts 合同信息
     * @return  是否成功
     * @description 批量插入收款信息
     * @date 2019/12/11 4:34 PM
     */
    public boolean insertBatch(List<Payment> payments, Contracts contracts) {
        //设置合同付款的合同id
        payments.forEach(entry -> {
            entry.setContractsId(contracts.getId());
            entry.setCode("PA" + DateUtils.generateCode());
            entry.setType(contracts.getType());
        });
        //批量插入付款信息
        return saveBatch(payments);
    }

    /**
     * @param paymentList 收款信息列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 批量修改收款信息
     * @date 2019/12/11 3:50 PM
     */
    public boolean modifyBatchById(List<Payment> paymentList) {
        return updateBatchById(paymentList);
    }

    /**
     * @param contractId 合同id
     * @return paymentList 付款列表
     * @description 查询付款列表
     * @date 2020/1/17 2:57 下午
     */
    public List<Payment> paymentList(Long contractId) {
        QueryWrapper<Payment> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("contracts_id", contractId);
        return list(queryWrapper);
    }
}
